@@ -1,11 +1,11 @@
-# $Id: Session.pm,v 1.81 2002/10/20 20:55:20 rcaputo Exp $
+# $Id: Session.pm,v 1.83 2003/01/01 04:24:40 rcaputo Exp $
 
 package POE::Session;
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = (qw($Revision: 1.81 $ ))[1];
+$VERSION = (qw($Revision: 1.83 $ ))[1];
 
 use Carp qw(carp croak);
 use POSIX qw(ENOSYS);
@@ -410,7 +410,9 @@ sub create {
       croak "the array for $param_name has an odd number of elements"
         if (@$param_value & 1);
 
-      while (my ($package, $handlers) = splice(@$param_value, 0, 2)) {
+      # Copy the parameters so they aren't destroyed.
+      my @param_value = @$param_value;
+      while (my ($package, $handlers) = splice(@param_value, 0, 2)) {
 
         # -><- What do we do if the package name has some sort of
         # blessing?  Do we use the blessed thingy's package, or do we
@@ -454,8 +456,10 @@ sub create {
       croak "the array for $param_name has an odd number of elements"
         if (@$param_value & 1);
 
-      while (@$param_value) {
-        my ($object, $handlers) = splice @$param_value => 0, 2;
+      # Copy the parameters so they aren't destroyed.
+      my @param_value = @$param_value;
+      while (@param_value) {
+        my ($object, $handlers) = splice(@param_value, 0, 2);
 
         # Verify that the object is an object.  This may catch simple
         # mistakes; or it may be overkill since it already checks that
@@ -1336,7 +1340,7 @@ this:
 
   sub put_stuff {
     my @stuff_to_put = @_;
-    $poe_kernel->get_active_session()->heap()->{wheel}->put( @stuff_to_put );
+    $poe_kernel->get_active_session()->get_heap()->{wheel}->put(@stuff);
   }
 
   sub some_state {
